// Copyright (C) 1999-2013
// Smithsonian Astrophysical Observatory, Cambridge, MA, USA
// For conditions of distribution and use, see copyright notice in "copyright"

#include <iostream>
#include <sstream>
#include <iomanip>
using namespace std;

#include <tkMacOSXPrivate.h>
#include <tkMacOSXFont.h>

#include "vector.h"

void XDrawRotString(Display* display, Drawable drawable, GC gc, 
		   Vector& v, double angle, const char* text, 
		   Tk_Font font)
{
  MacDrawable* macWin = (MacDrawable*)drawable;

  TkMacOSXDrawingContext dc;
  TkMacOSXSetupDrawingContext(drawable, gc, 1, &dc);

  // undo the xform in TkMacOSXSetupDrawingContext
  CGContextConcatCTM(dc.context, CGAffineTransformMake(1.0, 0.0, 0.0,
		 -1.0, 0.0, dc.portBounds.size.height));

  CGAffineTransform mm = CGAffineTransformMakeRotation(angle);
  CGContextSetTextMatrix(dc.context, mm);

  // map to window space
  Vector center = 
	Vector(dc.portBounds.size.width/2., dc.portBounds.size.height/2.);
  Vector vv = v *
    Translate(-center) *
    Translate(macWin->xOff, macWin->yOff) *
    FlipY() *
    Translate(center);

  // draw it
  TkFont* fontPtr = (TkFont*)font;
  Tcl_DString ds;
  Tcl_DStringInit(&ds);
  int size = Tk_PostscriptFontName(font, &ds);
  CGContextSelectFont(dc.context, Tcl_DStringValue(&ds), 
		      size, kCGEncodingMacRoman);
  Tcl_DStringFree(&ds);
  CGContextShowTextAtPoint (dc.context, vv[0], vv[1], text, strlen(text));

  // done
  TkMacOSXRestoreDrawingContext(&dc);
}
