/*
 * bltInt.h --
 *
 *	Copyright 1993-2004 George A Howlett.
 *
 *	Permission is hereby granted, free of charge, to any person obtaining
 *	a copy of this software and associated documentation files (the
 *	"Software"), to deal in the Software without restriction, including
 *	without limitation the rights to use, copy, modify, merge, publish,
 *	distribute, sublicense, and/or sell copies of the Software, and to
 *	permit persons to whom the Software is furnished to do so, subject to
 *	the following conditions:
 *
 *	The above copyright notice and this permission notice shall be
 *	included in all copies or substantial portions of the Software.
 *
 *	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *	NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 *	LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 *	OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 *	WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef _BLT_INT_H
#define _BLT_INT_H

#include <tcl.h>
#ifdef USE_TCL_STUBS
#include <tclInt.h>
#endif

#include <tk.h>
#ifdef USE_TK_STUBS
#include <tkInt.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <errno.h>
#include <ctype.h>
#include <memory.h>

#define TRUE 	1
#define FALSE 	0

/*
 *---------------------------------------------------------------------------
 *
 * Blt_InitCmdSpec --
 *
 *---------------------------------------------------------------------------
 */
typedef struct {
    const char *name;			/* Name of command */
    Tcl_ObjCmdProc *cmdProc;
    Tcl_CmdDeleteProc *cmdDeleteProc;
    ClientData clientData;
} Blt_InitCmdSpec;

extern int Blt_DictionaryCompare (const char *s1, const char *s2);

/* ---------------------------------------------------------------- */

#define COUNT_NNEG		0
#define COUNT_POS		1
#define COUNT_ANY		2

#define RGB_ANTIQUEWHITE1	"#ffefdb"
#define RGB_BISQUE1		"#ffe4c4"
#define RGB_BISQUE2		"#eed5b7"
#define RGB_BISQUE3		"#cdb79e"
#define RGB_BLACK		"#000000"
#define RGB_BLUE		"#0000ff"
#define RGB_GREEN		"#00ff00"
#define RGB_GREY		"#b0b0b0"
#define RGB_GREY15		"#262626"
#define RGB_GREY20		"#333333"
#define RGB_GREY25		"#404040"
#define RGB_GREY30		"#4d4d4d"
#define RGB_GREY35		"#595959"
#define RGB_GREY40		"#666666"
#define RGB_GREY50		"#7f7f7f"
#define RGB_GREY64		"#a3a3a3"
#define RGB_GREY70		"#b3b3b3"
#define RGB_GREY75		"#bfbfbf"
#define RGB_GREY77		"#c3c3c3"
#define RGB_GREY82		"#d1d1d1"
#define RGB_GREY85		"#d9d9d9"
#define RGB_GREY90		"#e5e5e5"
#define RGB_GREY93		"#ececec"
#define RGB_GREY95		"#f2f2f2"
#define RGB_GREY97		"#f7f7f7"
#define RGB_LIGHTBLUE0		"#e4f7ff"
#define RGB_LIGHTBLUE00		"#D9F5FF"
#define RGB_LIGHTBLUE1		"#bfefff"
#define RGB_LIGHTBLUE2		"#b2dfee"
#define RGB_LIGHTSKYBLUE1	"#b0e2ff"
#define RGB_MAROON		"#b03060"
#define RGB_NAVYBLUE		"#000080"
#define RGB_PINK		"#ffc0cb"
#define RGB_BISQUE1		"#ffe4c4"
#define RGB_RED			"#ff0000"
#define RGB_RED3		"#cd0000"
#define RGB_WHITE		"#ffffff"
#define RGB_YELLOW		"#ffff00"
#define RGB_SKYBLUE4		"#4a708b"

#define STD_NORMAL_BACKGROUND	RGB_GREY85
#define STD_ACTIVE_BACKGROUND	RGB_GREY90
#define STD_SELECT_BACKGROUND	RGB_SKYBLUE4
#define STD_SELECT_FOREGROUND	RGB_WHITE

#define STD_ACTIVE_BG_MONO	RGB_BLACK
#define STD_ACTIVE_FOREGROUND	RGB_BLACK
#define STD_ACTIVE_FG_MONO	RGB_WHITE
#define STD_BORDERWIDTH 	"2"
#define STD_FONT_HUGE		"{Sans Serif} 18"
#define STD_FONT_LARGE		"{Sans Serif} 14"
#define STD_FONT_MEDIUM		"{Sans Serif} 11"
#define STD_FONT_NORMAL		"{Sans Serif} 9"
#define STD_FONT_SMALL		"{Sans Serif} 8"
#define	STD_FONT_NUMBERS	"Math 8"
#define STD_FONT		STD_FONT_NORMAL
#define STD_INDICATOR_COLOR	RGB_RED3
#define STD_NORMAL_BG_MONO	RGB_WHITE
#define STD_NORMAL_FOREGROUND	RGB_BLACK
#define STD_NORMAL_FG_MONO	RGB_BLACK
#define STD_SELECT_BG_MONO	RGB_BLACK
#define STD_SELECT_BORDERWIDTH	"2"
#define STD_SELECT_FG_MONO	RGB_WHITE
#define STD_SHADOW_MONO		RGB_BLACK
#define STD_SELECT_FONT_HUGE	"{Sans Serif} 18 Bold"
#define STD_SELECT_FONT_LARGE	"{Sans Serif} 14 Bold"
#define STD_SELECT_FONT_MEDIUM	"{Sans Serif} 11 Bold"
#define STD_SELECT_FONT_NORMAL	"{Sans Serif} 9 Bold"
#define STD_SELECT_FONT_SMALL	"{Sans Serif} 8 Bold"
#define STD_SELECT_FONT		STD_SELECT_FONT_NORMAL
#define STD_DISABLED_FOREGROUND	RGB_GREY70
#define STD_DISABLED_BACKGROUND	RGB_GREY90

#define LineWidth(w)	(((w) > 1) ? (w) : 0)

#ifdef TCL_UTF_MAX
#  define HAVE_UTF	1
#else
#  define HAVE_UTF	0
#endif /* TCL_UTF_MAX */

#ifndef TK_RELIEF_SOLID
#  define TK_RELIEF_SOLID		TK_RELIEF_FLAT
#endif

typedef int (QSortCompareProc) (const void *, const void *);

/*
 *---------------------------------------------------------------------------
 *
 * Point2 --
 *
 *	2-D coordinate.
 *
 *---------------------------------------------------------------------------
 */
typedef struct {
    double x, y;
} Point2d;

typedef struct {
    float x, y;
} Point2f;

typedef struct {
    int x, y;
} Point2i;

typedef struct {
    size_t nValues; 
    void *values;
} Array;

typedef struct {
    Point2d p, q;		/* The two end points of the segment. */
} Segment2d;

typedef struct {
    short int width, height;
} Dim2D;

typedef struct {
    float left, right, top, bottom;
} Region2f;

typedef struct {
    double left, right, top, bottom;
} Region2d;

#define PointInRegion(e,x,y) \
	(((x) <= (e)->right) && ((x) >= (e)->left) && \
	 ((y) <= (e)->bottom) && ((y) >= (e)->top))

#define PointInRectangle(r,x0,y0) \
	(((x0) <= (int)((r)->x + (r)->width - 1)) && ((x0) >= (int)(r)->x) && \
	 ((y0) <= (int)((r)->y + (r)->height - 1)) && ((y0) >= (int)(r)->y))

/*-------------------------------------------------------------------------------
 *
 * ColorPair --
 *
 *	Holds a pair of foreground, background colors.
 *
 *---------------------------------------------------------------------------
 */
typedef struct {
    XColor *fgColor, *bgColor;
} ColorPair;

#define COLOR_NONE		(XColor *)0
#define COLOR_DEFAULT		(XColor *)1
#define COLOR_ALLOW_DEFAULTS	1

extern void strtolower(char*);

extern void Blt_FreeColorPair (ColorPair *pairPtr);

extern int Blt_ExprDoubleFromObj (Tcl_Interp *interp, Tcl_Obj *objPtr, 
	double *valuePtr);

extern int Blt_ExprIntFromObj (Tcl_Interp *interp, Tcl_Obj *objPtr, 
	int *valuePtr);

extern const char *Blt_Itoa(int value);

extern const char *Blt_Ltoa(long value);

extern const char *Blt_Utoa(unsigned int value);

extern const char *Blt_Dtoa(Tcl_Interp *interp, double value);

extern int Blt_InitCmd (Tcl_Interp *interp, const char *namespace, 
	Blt_InitCmdSpec *specPtr);

extern int Blt_InitCmds (Tcl_Interp *interp, const char *namespace, 
	Blt_InitCmdSpec *specPtr, int nCmds);

extern int Blt_GlobalEvalObjv(Tcl_Interp *interp, int objc, 
	Tcl_Obj *const *objv);

extern int Blt_GetDoubleFromString(Tcl_Interp *interp, const char *s, 
	double *valuePtr);
extern int Blt_GetDoubleFromObj(Tcl_Interp *interp, Tcl_Obj *objPtr, 
	double *valuePtr);

extern int Blt_LineRectClip(Region2d *regionPtr, Point2d *p, Point2d *q);

extern int Blt_SimplifyLine (Point2d *origPts, int low, int high, 
	double tolerance, int *indices);

extern GC Blt_GetPrivateGC(Tk_Window tkwin, unsigned long gcMask,
	XGCValues *valuePtr);

extern GC Blt_GetPrivateGCFromDrawable(Display *display, Drawable drawable, 
	unsigned long gcMask, XGCValues *valuePtr);

extern void Blt_FreePrivateGC(Display *display, GC gc);

extern int Blt_GetWindowFromObj(Tcl_Interp *interp, Tcl_Obj *objPtr, 
	Window *windowPtr);

extern Window Blt_GetParentWindow(Display *display, Window window);

extern int Blt_GetCountFromObj(Tcl_Interp *interp, Tcl_Obj *objPtr, 
	int check, long *valuePtr);

extern const char *Blt_NameOfFill(int fill);

extern int Blt_GetXY (Tcl_Interp *interp, Tk_Window tkwin, 
	const char *string, int *xPtr, int *yPtr);

extern Point2d Blt_GetProjection (int x, int y, Point2d *p, Point2d *q);

extern Window Blt_GetParent (Display *display, Window tkwin);

extern void Blt_GetBoundingBox (int width, int height, float angle, 
	double *widthPtr, double *heightPtr, Point2d *points);

extern void Blt_TranslateAnchor (int x, int y, int width, int height, 
	Tk_Anchor anchor, int *transXPtr, int *transYPtr);

extern Point2d Blt_AnchorPoint (double x, double y, double width, 
	double height, Tk_Anchor anchor);

extern long Blt_MaxRequestSize (Display *display, size_t elemSize);

extern Window Blt_GetWindowId (Tk_Window tkwin);

extern void Blt_InitXRandrConfig(Tcl_Interp *interp);

extern int Blt_GetWindowRegion(Display *display, Window window, int *xPtr,
	int *yPtr, int *widthPtr, int *heightPtr);

extern int Blt_AdjustViewport (int offset, int worldSize, int windowSize, 
	int scrollUnits, int scrollMode);

extern void Blt_UpdateScrollbar(Tcl_Interp *interp, 
	Tcl_Obj *scrollCmdObjPtr, int first, int last, int width);

extern GC Blt_GetBitmapGC(Tk_Window tkwin);

#define Tk_RootWindow(tkwin) \
  RootWindow(Tk_Display(tkwin),Tk_ScreenNumber(tkwin))

#include "bltConfig.h"

extern Tcl_AppInitProc Blt_GraphCmdInitProc;
extern Tcl_AppInitProc Blt_VectorCmdInitProc;

extern int TclGetLong(Tcl_Interp *interp, const char *s, long *longPtr);

extern int sprintf_s(char *s, size_t size, const char *fmt, /*args*/ ...);

extern void Blt_Panic TCL_VARARGS(const char *, args);

extern void Blt_ScreenDPI(Tk_Window tkwin, unsigned int *xPtr, 
	unsigned int *yPtr);

extern char *Blt_Strdup(const char *string);

#endif /*_BLT_INT_H*/
